/* Blackfin processor list

   Copyright (C) 2005-2018 Free Software Foundation, Inc.
   Contributed by Analog Devices, Inc.

   This file is part of simulators.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/* First entry is the default model.  */
P(537)
P(504)
P(506)
P(512)
P(514)
P(516)
P(518)
P(522)
P(523)
P(524)
P(525)
P(526)
P(527)
P(531)
P(532)
P(533)
P(534)
/*P(535)*/
P(536)
P(538)
P(539)
P(542)
P(544)
P(547)
P(548)
P(549)
P(561)
P(592)
P(000)
