/* This testcase is part of GDB, the GNU debugger.

   Copyright 2016-2018 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

#include <signal.h>
#include <string.h>
#include <unistd.h>

static int counter = 0;

static void
handler (int sig)
{
  counter++;
}

static int iterations = 3;

static void
start (int pid)
{
  int i;

  for (i = 0; i < iterations; i++)
    {
      kill (pid, SIGABRT);
    }
}

static void
end (void)
{}

int
main (void)
{
  struct sigaction act;
  int i;

  memset (&act, 0, sizeof act);
  act.sa_handler = handler;
  act.sa_flags = SA_NODEFER;
  sigaction (SIGABRT, &act, NULL);

  for (i = 0; i < 3; i++)
    {
      kill (getpid (), SIGABRT);
    }

  counter = 0;
  start (getpid ());

  end ();
  return 0;
}
