/* Copyright 2007-2018 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/* Author: Paul N. Hilfinger, AdaCore Inc. */

enum Enum1 { Val10=10, Val11, Val12 };

struct Struct1
{
      int x, y; 
};

int f1 (Struct1& s, Enum1& e, int& i)
{
  return s.x;			/* Set breakpoint marker here.  */
}

Struct1 s1 = { 13, 19 };

int i1 = 23;

Enum1 e1 = Val11;

int main(void) 
{

  f1 (s1, e1, i1);

}
