// { dg-do run { xfail *-*-* } }
// { dg-options "-std=gnu++17 -O0" }
// { dg-require-debug-mode "" }

// Copyright (C) 2013-2018 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 3, or (at your option)
// any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License along
// with this library; see the file COPYING3.  If not see
// <http://www.gnu.org/licenses/>.

#include <string_view>

// libstdc++/21674
// NB: Should work without any inlining or optimizations (ie. -O0).
int
main()
{
  typedef std::wstring_view string_view_type;
  string_view_type s;
  s[0]; // abort
}
